@@ -16,20 +16,23 @@ use std::ops::Range;
 use rand::prelude::*;
 use crate::gapbuffer::GapBuffer;
 use crate::utils::*;
+// use crate::params::*;
 
 // Must be <= UINT16_MAX. Benchmarking says this is pretty close to optimal
 // (tested on a mac using clang 4.0 and x86_64).
 //const NODE_SIZE: usize = 136;
 
 // The likelyhood (out of 256) a node will have height (n+1) instead of n
-const BIAS: u8 = 100;
+const BIAS: u8 = 65;
+// const BIAS: u8 = XX_BIAS;
 
 // The rope will become less efficient after the string is 2 ^ ROPE_MAX_HEIGHT nodes.
 
 #[cfg(debug_assertions)]
 const NODE_STR_SIZE: usize = 10;
 #[cfg(not(debug_assertions))]
-const NODE_STR_SIZE: usize = 100;
+const NODE_STR_SIZE: usize = 392;
+// const NODE_STR_SIZE: usize = XX_SIZE;
 
 const MAX_HEIGHT: usize = 20;//NODE_STR_SIZE / mem::size_of::<SkipEntry>();
 const MAX_HEIGHT_U8: u8 = MAX_HEIGHT as u8;
@@ -373,6 +376,22 @@ impl JumpRope {
         // The insertion offset into the destination node.
         let offset: usize = cursor.0[0].skip_chars;
         let mut e = cursor.here_ptr();
+
+        // We might be able to insert the new data into the current node, depending on
+        // how big it is. We'll count the bytes, and also check that its valid utf8.
+        let num_inserted_bytes = contents.len();
+        let num_inserted_chars = count_chars(contents);
+
+        // Adding this short curcuit makes the code about 2% faster for 1% more code
+        if (*e).str.gap_start_chars as usize == offset && (*e).str.gap_len as usize >= num_inserted_bytes {
+            // Short circuit. If we can just insert all the content right here in the gap, do so.
+            (*e).str.insert_in_gap(contents);
+            cursor.update_offsets(self.head.height as usize, num_inserted_chars as isize);
+            cursor.move_within_node(self.head.height as usize, num_inserted_chars as isize);
+            self.num_bytes += num_inserted_bytes;
+            return;
+        }
+
         if offset > 0 {
             assert!(offset <= (*e).nexts()[0].skip_chars);
             // This could be faster, but its not a big deal.
@@ -385,11 +404,6 @@ impl JumpRope {
             // println!("{:?}", v);
         }
 
-        // We might be able to insert the new data into the current node, depending on
-        // how big it is. We'll count the bytes, and also check that its valid utf8.
-        let num_inserted_bytes = contents.len();
-        let num_inserted_chars = count_chars(contents);
-
         // Can we insert into the current node?
         let current_len_bytes = (*e).str.len_bytes();
         let mut insert_here = current_len_bytes + num_inserted_bytes <= NODE_STR_SIZE;
@@ -516,15 +530,8 @@ impl JumpRope {
             let height = (*node).height as usize;
             if removed < num_chars || std::ptr::eq(node, &self.head) {
                 // Just trim the node down.
-                // let s = (*e).as_str();
                 let s = &mut (*node).str;
-
-                // TODO: Consider making a remove_chars in GapBuffer.
-                let leading_bytes = s.count_bytes(offset);
-                s.move_gap(leading_bytes);
-                let removed_bytes = str_get_byte_offset(s.end_as_str(), removed);
-                s.remove_at_gap(removed_bytes);
-
+                let removed_bytes = s.remove_chars(offset, removed);
                 self.num_bytes -= removed_bytes;
 
                 for s in (*node).nexts_mut() {
@@ -579,6 +586,12 @@ impl<'a> From<&'a str> for JumpRope {
     }
 }
 
+impl From<String> for JumpRope {
+    fn from(str: String) -> Self {
+        JumpRope::new_from_str(&str)
+    }
+}
+
 impl PartialEq for JumpRope {
     // This is quite complicated. It would be cleaner to just write a bytes
     // iterator, then iterate over the bytes of both strings comparing along the
@@ -731,6 +744,7 @@ impl JumpRope {
                 assert!(!n.str.is_empty() || std::ptr::eq(n, &self.head));
                 assert!(n.height <= MAX_HEIGHT_U8);
                 assert!(n.height >= 1);
+                n.str.check();
 
                 assert_eq!(count_chars(n.as_str_1()) + count_chars(n.as_str_2()), n.num_chars());
                 for (i, entry) in iter[0..n.height as usize].iter_mut().enumerate() {
